@@ -1,3 +1,8 @@
+imports.gi.versions.Gio = '2.0';
+imports.gi.versions.GLib = '2.0';
+imports.gi.versions.Gtk = '3.0';
+imports.gi.versions.TelepathyGLib = '0.12';
+
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
@@ -34,6 +39,7 @@ const Application = new Lang.Class({
         GLib.set_prgname('polari');
         GLib.set_application_name('Polari');
         this._window = null;
+        this._pendingRequests = {};
     },
 
     vfunc_startup: function() {
@@ -55,6 +61,11 @@ const Application = new Lang.Class({
         this._chatroomManager = ChatroomManager.getDefault();
         this._accountsMonitor = AccountsMonitor.getDefault();
 
+        this._accountsMonitor.connect('account-removed', Lang.bind(this,
+            function(am, account) {
+                this._removeSavedChannelsForAccount(account);
+            }));
+
         this._settings = new Gio.Settings({ schema: 'org.gnome.polari' });
 
         this.pasteManager = new PasteManager.PasteManager();
@@ -68,11 +79,9 @@ const Application = new Lang.Class({
         let actionEntries = [
           { name: 'room-menu',
             activate: Lang.bind(this, this._onToggleAction),
-            create_hook: Lang.bind(this, this._accountActionsCreateHook),
             state: GLib.Variant.new('b', false) },
           { name: 'show-join-dialog',
             activate: Lang.bind(this, this._onShowJoinDialog),
-            create_hook: Lang.bind(this, this._accountActionsCreateHook),
             accel: '<Primary>n' },
           { name: 'show-message-user-dialog',
             activate: Lang.bind(this, this._onShowMessageUserDialog),
@@ -117,7 +126,9 @@ const Application = new Lang.Class({
           { name: 'first-room',
             accel: '<Primary>Home' },
           { name: 'last-room',
-            accel: '<Primary>End' }
+            accel: '<Primary>End' },
+          { name: 'nth-room',
+            parameter_type: GLib.VariantType.new('i') }
         ];
         actionEntries.forEach(Lang.bind(this,
             function(actionEntry) {
@@ -140,9 +151,15 @@ const Application = new Lang.Class({
                 this.add_action(action);
         }));
 
+        for (let i = 1; i < 10; i++)
+            this.add_accelerator('<Alt>' + i, 'app.nth-room',
+                                 GLib.Variant.new('i', i));
+
         this._window = new MainWindow.MainWindow(this);
         this._window.window.connect('destroy', Lang.bind(this,
             function() {
+                for (let id in this._pendingRequests)
+                    this._pendingRequests[id].cancellable.cancel();
                 this.emitJS('prepare-shutdown');
             }));
 
@@ -190,23 +207,17 @@ const Application = new Lang.Class({
 
     _updateUserListAction: function(action) {
         let room = this._chatroomManager.getActiveRoom();
-        action.enabled = room && room.channel.handle_type == Tp.HandleType.ROOM;
+        action.enabled = room && room.type == Tp.HandleType.ROOM && room.channel;
     },
 
     _userListCreateHook: function(action) {
-        this._chatroomManager.connect('active-changed', Lang.bind(this,
+        this._chatroomManager.connect('active-state-changed', Lang.bind(this,
             function() {
                 this._updateUserListAction(action);
             }));
         action.connect('notify::enabled', function() {
-            if (action.enabled) {
-                if (action._previousState)
-                    action.change_state(GLib.Variant.new('b', true));
-                delete action._previousState;
-            } else {
-                action._previousState = action.state.get_boolean();
+            if (!action.enabled)
                 action.change_state(GLib.Variant.new('b', false));
-            }
         });
         this._updateUserListAction(action);
     },
@@ -268,6 +279,17 @@ const Application = new Lang.Class({
                                  GLib.Variant.new('aa{sv}', savedChannels));
     },
 
+    _removeSavedChannelsForAccount: function(account) {
+        let savedChannels = this._settings.get_value('saved-channel-list').deep_unpack();
+        let accountPath = GLib.Variant.new('s', account.get_object_path());
+
+        let savedChannels = savedChannels.filter(function(a) {
+            return !a.account.equal(accountPath);
+        });
+        this._settings.set_value('saved-channel-list',
+                                 GLib.Variant.new('aa{sv}', savedChannels));
+    },
+
     _updateAccountName: function(account, name, callback) {
         let sv = { account: GLib.Variant.new('s', name) };
         let asv = GLib.Variant.new('a{sv}', sv);
@@ -279,17 +301,29 @@ const Application = new Lang.Class({
         let factory = Tp.AccountManager.dup().get_factory();
         let account = factory.ensure_account(accountPath, []);
 
-        if (!account.enabled)
+        if (!account.enabled) {
+            // if we are requesting a channel for a disabled account, we
+            // are restoring saved channels; if the account has also never
+            // been online, it was removed since the channel was saved
+            if (!account.has_been_online)
+                this._removeSavedChannelsForAccount(account);
             return;
+        }
+
+        let roomId = Polari.create_room_id(account,  targetId, targetType);
 
         let requestData = {
           account: account,
           targetHandleType: targetType,
           targetId: targetId,
+          roomId: roomId,
+          cancellable: new Gio.Cancellable(),
           time: time,
           retry: 0,
           originalNick: account.nickname };
 
+        this._pendingRequests[roomId] = requestData;
+
         this._ensureChannel(requestData);
     },
 
@@ -300,7 +334,7 @@ const Application = new Lang.Class({
         req.set_target_id(requestData.targetHandleType, requestData.targetId);
         req.set_delegate_to_preferred_handler(true);
         let preferredHandler = Tp.CLIENT_BUS_NAME_BASE + 'Polari';
-        req.ensure_channel_async(preferredHandler, null,
+        req.ensure_channel_async(preferredHandler, requestData.cancellable,
                                  Lang.bind(this,
                                            this._onEnsureChannel, requestData));
     },
@@ -332,12 +366,15 @@ const Application = new Lang.Class({
                     this._ensureChannel(requestData);
                 }));
             return;
+        } catch (e if e.matches(Tp.Error, Tp.Error.CANCELLED)) {
+            // interrupted by user request, don't log
         } catch (e) {
             logError(e, 'Failed to ensure channel');
         }
 
         if (requestData.retry > 0)
             this._updateAccountName(account, requestData.originalNick, null);
+        delete this._pendingRequests[requestData.roomId];
     },
 
     _onJoinRoom: function(action, parameter) {
@@ -358,18 +395,21 @@ const Application = new Lang.Class({
         let room = this._chatroomManager.getRoomById(roomId);
         if (!room)
             return;
-        if (!message.length)
-            message = _("Good Bye"); // TODO - our first setting?
-        room.channel.leave_async(reason, message, Lang.bind(this,
-            function(c, res) {
-                try {
-                    c.leave_finish(res);
-                } catch(e) {
-                    logError(e, 'Failed to leave channel');
-                }
-            }));
-        this._removeSavedChannel(room.channel.connection.get_account(),
-                                 room.channel.identifier);
+        if (this._pendingRequests[roomId]) {
+            this._pendingRequests[roomId].cancellable.cancel();
+        } else if (room.channel) {
+            if (!message.length)
+                message = _("Good Bye"); // TODO - our first setting?
+            room.channel.leave_async(reason, message, Lang.bind(this,
+                function(c, res) {
+                    try {
+                        c.leave_finish(res);
+                    } catch(e) {
+                        logError(e, 'Failed to leave channel');
+                    }
+                }));
+        }
+        this._removeSavedChannel(room.account, room.channel_name);
     },
 
     _onLeaveCurrentRoom: function() {
@@ -386,18 +426,28 @@ const Application = new Lang.Class({
     },
 
     _onListConnections: function() {
-        let dialog = new Connections.ConnectionsDialog();
-        dialog.widget.show();
-        dialog.widget.connect('response',
-            function(widget) {
+        if (this._connectionsDialog) {
+            this._connectionsDialog.widget.present();
+            return;
+        }
+
+        this._connectionsDialog = new Connections.ConnectionsDialog();
+        this._connectionsDialog.widget.show();
+        this._connectionsDialog.widget.connect('response',
+            Lang.bind(this, function(widget) {
                 widget.destroy();
-            });
+                this._connectionsDialog = null;
+            }));
     },
 
     _onShowPreferences: function() {
     },
 
     _onShowAbout: function() {
+        if (this._aboutDialog) {
+            this._aboutDialog.present();
+            return;
+        }
         let aboutParams = {
             authors: [
                 'Florian M' + String.fromCharCode(0x00FC) // ü
@@ -419,11 +469,12 @@ const Application = new Lang.Class({
             modal: true
         };
 
-        let dialog = new Gtk.AboutDialog(aboutParams);
-        dialog.show();
-        dialog.connect('response', function() {
-            dialog.destroy();
-        });
+        this._aboutDialog = new Gtk.AboutDialog(aboutParams);
+        this._aboutDialog.show();
+        this._aboutDialog.connect('response', Lang.bind(this, function() {
+            this._aboutDialog.destroy();
+            this._aboutDialog = null;
+        }));
     },
 
     _onQuit: function() {
