@@ -8,13 +8,14 @@ const Tpl = imports.gi.TelepathyLogger;
 
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
-const Notify = imports.notify;
 const Utils = imports.utils;
 
 const MAX_NICK_CHARS = 8;
 const IGNORE_STATUS_TIME = 5;
 const TP_CURRENT_TIME = GLib.MAXUINT32;
 
+const SCROLL_TIMEOUT = 100; // ms
+
 const TIMESTAMP_INTERVAL = 300; // seconds of inactivity after which to
                                 // insert a timestamp
 
@@ -92,14 +93,13 @@ const ChatView = new Lang.Class({
         this._pending = {};
         this._pendingLogs = [];
 
-        let isRoom = room.channel.has_interface(Tp.IFACE_CHANNEL_INTERFACE_GROUP);
-        let account = room.channel.connection.get_account();
+        let isRoom = room.type == Tp.HandleType.ROOM;
         let target = new Tpl.Entity({ type: isRoom ? Tpl.EntityType.ROOM
                                                    : Tpl.EntityType.CONTACT,
-                                      identifier: room.channel.identifier });
+                                      identifier: room.channel_name });
         let logManager = Tpl.LogManager.dup_singleton();
         this._logWalker =
-            logManager.walk_filtered_events(account, target,
+            logManager.walk_filtered_events(room.account, target,
                                             Tpl.EventTypeMask.TEXT, null);
 
         this._fetchingBacklog = true;
@@ -109,25 +109,16 @@ const ChatView = new Lang.Class({
         let adj = this.widget.vadjustment;
         this._scrollBottom = adj.upper - adj.page_size;
 
-        let app = Gio.Application.get_default();
-        app.pasteManager.addWidget(this._view);
+        this._app = Gio.Application.get_default();
+        this._app.pasteManager.addWidget(this._view);
 
         this._linkCursor = Gdk.Cursor.new(Gdk.CursorType.HAND1);
 
-        let channelSignals = [
-            { name: 'message-received',
-              handler: Lang.bind(this, this._insertTpMessage) },
-            { name: 'message-sent',
-              handler: Lang.bind(this, this._insertTpMessage) },
-            { name: 'pending-message-removed',
-              handler: Lang.bind(this, this._pendingMessageRemoved) }
-        ];
         this._channelSignals = [];
-        channelSignals.forEach(Lang.bind(this, function(signal) {
-            this._channelSignals.push(room.channel.connect(signal.name, signal.handler));
-        }));
 
         let roomSignals = [
+            { name: 'notify::channel',
+              handler: Lang.bind(this, this._onChannelChanged) },
             { name: 'member-renamed',
               handler: Lang.bind(this, this._onMemberRenamed) },
             { name: 'member-disconnected',
@@ -145,12 +136,6 @@ const ChatView = new Lang.Class({
         roomSignals.forEach(Lang.bind(this, function(signal) {
             this._roomSignals.push(room.connect(signal.name, signal.handler));
         }));
-
-        room.channel.dup_pending_messages().forEach(Lang.bind(this,
-            function(message) {
-                this._insertTpMessage(room, message);
-            }));
-        this._checkMessages();
     },
 
     _createTags: function() {
@@ -224,7 +209,7 @@ const ChatView = new Lang.Class({
     },
 
     _createWidget: function() {
-        this.widget = new Gtk.ScrolledWindow();
+        this.widget = new Gtk.ScrolledWindow({ vexpand: true });
         this.widget.hscrollbar_policy = Gtk.PolicyType.NEVER;
         this.widget.resize_mode = Gtk.ResizeMode.QUEUE;
 
@@ -248,7 +233,7 @@ const ChatView = new Lang.Class({
                             Lang.bind(this, this._updateToplevel));
         this.widget.connect('scroll-event', Lang.bind(this ,this._onScroll));
         this.widget.vadjustment.connect('value-changed',
-                                 Lang.bind(this, this._checkMessages));
+                                        Lang.bind(this, this._onValueChanged));
         this.widget.vadjustment.connect('changed',
                                  Lang.bind(this, this._updateScroll));
         this._view.connect('button-release-event',
@@ -349,14 +334,15 @@ const ChatView = new Lang.Class({
 
     _updateScroll: function() {
         let adj = this.widget.vadjustment;
-        if (this._nPending == 0) {
-            if (adj.value == this._scrollBottom)
+        if (adj.value == this._scrollBottom) {
+            if (this._nPending == 0) {
                 adj.value = adj.upper - adj.page_size;
-        } else if (!this._active) {
-            let id = Object.keys(this._pending).sort(function(a, b) {
-                return a - b;
-            })[0];
-            this._view.scroll_mark_onscreen(this._pending[id]);
+            } else {
+                let id = Object.keys(this._pending).sort(function(a, b) {
+                    return a - b;
+                })[0];
+                this._view.scroll_mark_onscreen(this._pending[id]);
+            }
         }
         this._scrollBottom = adj.upper - adj.page_size;
     },
@@ -380,10 +366,23 @@ const ChatView = new Lang.Class({
         return false;
     },
 
+    _onValueChanged: function() {
+        if (this._valueChangedId)
+            return;
+
+        this._valueChangedId = Mainloop.timeout_add(SCROLL_TIMEOUT, Lang.bind(this,
+            function() {
+                this._checkMessages();
+                this._valueChangedId = 0;
+                return false;
+            }));
+    },
+
     _pendingMessageRemoved: function(channel, message) {
         let [id,] = message.get_pending_message_id();
         if (this._pending[id])
             this._view.buffer.delete_mark(this._pending[id]);
+        this._app.withdraw_notification('pending-message-' + id);
         delete this._pending[id];
     },
 
@@ -441,7 +440,7 @@ const ChatView = new Lang.Class({
     },
 
     _checkMessages: function() {
-        if (!this._active || !this._toplevelFocus)
+        if (!this._active || !this._toplevelFocus || !this._room.channel)
             return;
 
         this._needsIndicator = true;
@@ -466,6 +465,38 @@ const ChatView = new Lang.Class({
         }
     },
 
+    _onChannelChanged: function() {
+        if (!this._room.channel) {
+            this._channelSignals = [];
+            return;
+        }
+
+        for (let i = 0; i < this._channelSignals.length; i++)
+            this._room.channel.disconnect(this._channelSignals[i]);
+        this._channelSignals = [];
+
+        let channelSignals = [
+            { name: 'message-received',
+              handler: Lang.bind(this, this._insertTpMessage) },
+            { name: 'message-sent',
+              handler: Lang.bind(this, this._insertTpMessage) },
+            { name: 'pending-message-removed',
+              handler: Lang.bind(this, this._pendingMessageRemoved) }
+        ];
+        channelSignals.forEach(Lang.bind(this, function(signal) {
+            this._channelSignals.push(this._room.channel.connect(signal.name, signal.handler));
+        }));
+
+        this._room.channel.dup_pending_messages().forEach(Lang.bind(this,
+            function(message) {
+                let [id, ] = message.get_pending_message_id();
+                if (this._pending[id])
+                    return;
+                this._insertTpMessage(this._room, message);
+            }));
+        this._checkMessages();
+    },
+
     _onMemberRenamed: function(room, oldMember, newMember) {
         this._insertStatus(_("%s is now known as %s").format(oldMember.alias,
                                                              newMember.alias));
@@ -522,30 +553,68 @@ const ChatView = new Lang.Class({
         let daysAgo = now.difference(date) / GLib.TIME_SPAN_DAY;
 
         let format;
-        // Show only the hour if date is on today
-        if(daysAgo < 1){
-            format = "%H:%M";
-        }
-        // Show the word "Yesterday" and time if date is on yesterday
-        else if(daysAgo <2){
-            /* Translators: this is the word "Yesterday" followed by a time string. i.e. "Yesterday, 14:30"*/
-            // xgettext:no-c-format
-            format = _("Yesterday, %H:%M");
-        }
-        // Show a week day and time if date is in the last week
-        else if (daysAgo < 7) {
-            /* Translators: this is the week day name followed by a time string. i.e. "Monday, 14:30*/
-            // xgettext:no-c-format
-            format = _("%A, %H:%M");
+        let desktopSettings = new Gio.Settings({ schema: 'org.gnome.desktop.interface' });
+        let clockFormat = desktopSettings.get_string('clock-format');
 
-        } else if (date.get_year() == now.get_year()) {
-            /* Translators: this is the month name and day number followed by a time string. i.e. "May 25, 14:30"*/
-            // xgettext:no-c-format
-            format = _("%B %d, %H:%M");
-        } else {
-            /* Translators: this is the month name, day number, year number followed by a time string. i.e. "May 25 2012, 14:30"*/
-            // xgettext:no-c-format
-            format = _("%B %d %Y, %H:%M");
+        switch (clockFormat) {
+            case '24h':
+                if(daysAgo < 1) { // today
+                    /* Translators: Time in 24h format */
+                    format = _("%H\u2236%M");
+                } else if(daysAgo <2) { // yesterday
+                    /* Translators: this is the word "Yesterday" followed by a
+                     time string in 24h format. i.e. "Yesterday, 14:30" */
+                    // xgettext:no-c-format
+                    format = _("Yesterday, %H\u2236%M");
+                } else if (daysAgo < 7) { // this week
+                    /* Translators: this is the week day name followed by a time
+                     string in 24h format. i.e. "Monday, 14:30" */
+                    // xgettext:no-c-format
+                    format = _("%A, %H\u2236%M");
+                } else if (date.get_year() == now.get_year()) { // this year
+                    /* Translators: this is the month name and day number
+                     followed by a time string in 24h format.
+                     i.e. "May 25, 14:30" */
+                    // xgettext:no-c-format
+                    format = _("%B %d, %H\u2236%M");
+                } else { // before this year
+                    /* Translators: this is the month name, day number, year
+                     number followed by a time string in 24h format.
+                     i.e. "May 25 2012, 14:30" */
+                    // xgettext:no-c-format
+                    format = _("%B %d %Y, %H\u2236%M");
+                }
+                break;
+        default:
+            // explicit fall-through
+            case '12h':
+                if(daysAgo < 1) { // today
+                    /* Translators: Time in 12h format */
+                    format = _("%l\u2236%M %p");
+                } else if(daysAgo <2) { // yesterday
+                    /* Translators: this is the word "Yesterday" followed by a
+                     time string in 12h format. i.e. "Yesterday, 2:30 pm" */
+                    // xgettext:no-c-format
+                    format = _("Yesterday, %l\u2236%M %p");
+                } else if (daysAgo < 7) { // this week
+                    /* Translators: this is the week day name followed by a time
+                     string in 12h format. i.e. "Monday, 2:30 pm" */
+                    // xgettext:no-c-format
+                    format = _("%A, %l\u2236%M %p");
+                } else if (date.get_year() == now.get_year()) { // this year
+                    /* Translators: this is the month name and day number
+                     followed by a time string in 12h format.
+                     i.e. "May 25, 2:30 pm" */
+                    // xgettext:no-c-format
+                    format = _("%B %d, %l\u2236%M %p");
+                } else { // before this year
+                    /* Translators: this is the month name, day number, year
+                     number followed by a time string in 12h format.
+                     i.e. "May 25 2012, 2:30 pm"*/
+                    // xgettext:no-c-format
+                    format = _("%B %d %Y, %l\u2236%M %p");
+                }
+                break;
         }
 
         return date.format(format);
@@ -574,20 +643,17 @@ const ChatView = new Lang.Class({
 
         if (message.shouldHighlight && !this._toplevelFocus) {
             let summary = '%s %s'.format(this._room.display_name, message.nick);
-            let notification = new Notify.Notification(summary, message.text);
+            let notification = new Gio.Notification();
+            notification.set_title(summary);
+            notification.set_body(message.text);
 
-            let account = this._room.channel.connection.get_account();
+            let account = this._room.account;
             let param = GLib.Variant.new('(ssu)',
                                          [ account.get_object_path(),
-                                           this._room.channel.identifier,
+                                           this._room.channel_name,
                                            TP_CURRENT_TIME ]);
-            notification.addAction('default', 'default');
-            notification.connect('action-invoked', function() {
-                let app = Gio.Application.get_default();
-                let action = app.lookup_action('join-room');
-                action.activate(param);
-            });
-            notification.show();
+            notification.set_default_action_and_target('app.join-room', param);
+            this._app.send_notification('pending-message-' + id, notification);
         }
 
         let buffer = this._view.get_buffer();
