@@ -7,18 +7,15 @@ const Tp = imports.gi.TelepathyGLib;
 const AccountsMonitor = imports.accountsMonitor;
 const AppNotifications = imports.appNotifications;
 const ChatroomManager = imports.chatroomManager;
-const ChatView = imports.chatView;
-const IrcParser = imports.ircParser;
 const JoinDialog = imports.joinDialog;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const MessageDialog = imports.messageDialog;
 const RoomList = imports.roomList;
-const TabCompletion = imports.tabCompletion;
+const RoomStack = imports.roomStack;
 const UserList = imports.userList;
 const Utils = imports.utils;
 
-const MAX_NICK_UPDATE_TIME = 5; /* s */
 const CONFIGURE_TIMEOUT = 100; /* ms */
 
 
@@ -26,121 +23,54 @@ const MainWindow = new Lang.Class({
     Name: 'MainWindow',
 
     _init: function(app) {
-        let builder = new Gtk.Builder();
-        builder.add_from_resource('/org/gnome/polari/main-window.ui');
+        this._rooms = {};
+        this._entries = {};
 
-        this.window = builder.get_object('main_window');
-        this.window.application = app;
+        this._room = null;
+        this._settings = new Gio.Settings({ schema: 'org.gnome.polari' });
+        this._gtkSettings = Gtk.Settings.get_default();
 
-        let overlay = builder.get_object('overlay');
+        this._displayNameChangedId = 0;
+        this._topicChangedId = 0;
+        this._membersChangedId = 0;
+        this._configureId = 0;
 
-        overlay.add_overlay(app.notificationQueue.widget);
-        overlay.add_overlay(app.commandOutputQueue.widget);
+        this._createWidget(app);
 
-        this._ircParser = new IrcParser.IrcParser();
-
-        this._accountsMonitor = new AccountsMonitor.getDefault();
+        this._accountsMonitor = AccountsMonitor.getDefault();
         this._accountsMonitor.connect('account-status-changed',
                                       Lang.bind(this, this._onAccountChanged));
         this._accountsMonitor.connect('account-added',
                                       Lang.bind(this, this._onAccountChanged));
 
-        this._roomManager = new ChatroomManager.getDefault();
-        this._roomManager.connect('room-added',
-                                  Lang.bind(this, this._roomAdded));
-        this._roomManager.connect('room-removed',
-                                  Lang.bind(this, this._roomRemoved));
+        this._roomManager = ChatroomManager.getDefault();
         this._roomManager.connect('active-changed',
                                   Lang.bind(this, this._activeRoomChanged));
+        this._roomManager.connect('active-state-changed',
+                                  Lang.bind(this, this._updateUserListLabel));
 
-        this._rooms = {};
+        this._updateUserListLabel();
 
-        this._room = null;
-        this._settings = new Gio.Settings({ schema: 'org.gnome.polari' });
+        this._userListAction = app.lookup_action('user-list');
 
-        this._displayNameChangedId = 0;
-        this._topicChangedId = 0;
-        this._nicknameChangedId = 0;
-        this._configureId = 0;
-
-        this._titlebarRight = builder.get_object('titlebar_right');
-        this._titlebarLeft = builder.get_object('titlebar_left');
-
-        this._titleLabel = builder.get_object('title_label');
-        this._subtitleLabel = builder.get_object('subtitle_label');
-
-        this._selectionRevealer = builder.get_object('selection_toolbar_revealer');
-        this._showUserListButton = builder.get_object('show_user_list_button');
-        this._revealer = builder.get_object('room_list_revealer');
-        this._chatStack = builder.get_object('chat_stack');
-        this._inputArea = builder.get_object('main_input_area');
-        this._nickEntry = builder.get_object('nick_entry');
-        this._entry = builder.get_object('message_entry');
-
-        this._nickEntry.width_chars = ChatView.MAX_NICK_CHARS
-        this._completion = new TabCompletion.TabCompletion(this._entry);
-
-        let scroll = builder.get_object('room_list_scrollview');
-        this._roomList = new RoomList.RoomList();
-        scroll.add(this._roomList.widget);
-
-        let sidebar = builder.get_object('user_list_sidebar');
-        this._userListSidebar = new UserList.UserListSidebar();
-        sidebar.add(this._userListSidebar.widget);
-
-        let revealer = builder.get_object('user_list_revealer');
         app.connect('action-state-changed::user-list', Lang.bind(this,
             function(group, actionName, value) {
-                revealer.reveal_child = value.get_boolean();
+                this._userListPopover.widget.visible = value.get_boolean();
             }));
-        revealer.connect('notify::child-revealed', Lang.bind(this,
+        this._userListPopover.widget.connect('notify::visible', Lang.bind(this,
             function() {
-                this._userListSidebar.animateEntry = revealer.child_revealed;
+                if (!this._userListPopover.widget.visible)
+                    this._userListAction.change_state(GLib.Variant.new('b', false));
             }));
 
         this._selectionModeAction = app.lookup_action('selection-mode');
         this._selectionModeAction.connect('notify::state',
                     Lang.bind(this, this._onSelectionModeChanged));
 
-        this._userListAction = app.lookup_action('user-list');
+        this._gtkSettings.connect('notify::gtk-decoration-layout',
+                                  Lang.bind(this, this._updateDecorations));
+        this._updateDecorations();
 
-        this._entry.connect('activate', Lang.bind(this,
-            function() {
-                this._ircParser.process(this._entry.text);
-                this._entry.text = '';
-            }));
-        this._entry.connect('notify::is-focus', Lang.bind(this,
-            function() {
-                // HACK: force focus to the entry unless it was
-                //       moved by keynav or moved to another entry
-                if (this.window.get_focus() instanceof Gtk.Entry)
-                    return;
-                let device = Gtk.get_current_event_device();
-                if (!device || device.get_source() == Gdk.InputSource.KEYBOARD)
-                    return;
-                this._entry.grab_focus();
-            }));
-
-        this._nickEntry.connect('activate', Lang.bind(this,
-            function() {
-               if (this._nickEntry.text)
-                   this._setNick(this._nickEntry.text);
-               this._entry.grab_focus();
-            }));
-        this._nickEntry.connect('focus-out-event', Lang.bind(this,
-             function() {
-               this._nickEntry.text = '';
-               return false;
-            }));
-        this._nickEntry.connect_after('key-press-event', Lang.bind(this,
-            function(w, event) {
-                let [, keyval] = event.get_keyval();
-                if (keyval == Gdk.KEY_Escape) {
-                    this._entry.grab_focus();
-                    return true;
-                }
-                return false;
-            }));
         this.window.connect_after('key-press-event', Lang.bind(this,
             function(w, event) {
                 let [, keyval] = event.get_keyval();
@@ -172,8 +102,6 @@ const MainWindow = new Lang.Class({
         if (this._settings.get_boolean('window-maximized'))
             this.window.maximize();
 
-        this._updateSensitivity();
-
         this.window.show_all();
     },
 
@@ -235,6 +163,7 @@ const MainWindow = new Lang.Class({
     _onSelectionModeChanged: function() {
         let enabled = this._selectionModeAction.state.get_boolean();
         this._selectionRevealer.reveal_child = enabled;
+        this._joinMenuButton.visible = !enabled;
         this._showUserListButton.visible = !enabled;
         this._userListAction.enabled = !enabled;
 
@@ -257,25 +186,30 @@ const MainWindow = new Lang.Class({
         let app = Gio.Application.get_default();
         let notification = new AppNotifications.ConnectingNotification(account);
         app.notificationQueue.addNotification(notification);
+        app.mark_busy();
 
         account._connectingNotification = notification;
         notification.widget.connect('destroy',
             function() {
+                app.unmark_busy();
 		delete account._connectingNotification;
             });
     },
 
+    _updateDecorations: function() {
+        let layoutLeft = null;
+        let layoutRight = null;
 
-    _roomAdded: function(roomManager, room) {
-        let chatView = new ChatView.ChatView(room);
-        this._rooms[room.id] = chatView;
+        let layout = this._gtkSettings.gtk_decoration_layout;
+        if (layout) {
+            let split = layout.split(':');
 
-        this._chatStack.add_named(chatView.widget, room.id);
-    },
+            layoutLeft = split[0] + ':';
+            layoutRight = ':' + split[1];
+        }
 
-    _roomRemoved: function(roomManager, room) {
-        this._rooms[room.id].widget.destroy();
-        delete this._rooms[room.id];
+        this._titlebarLeft.set_decoration_layout(layoutLeft);
+        this._titlebarRight.set_decoration_layout(layoutRight);
     },
 
     _activeRoomChanged: function(manager, room) {
@@ -283,19 +217,15 @@ const MainWindow = new Lang.Class({
             this._room.disconnect(this._displayNameChangedId);
             this._room.disconnect(this._topicChangedId);
             this._room.disconnect(this._membersChangedId);
-            this._room.channel.connection.disconnect(this._nicknameChangedId);
         }
         this._displayNameChangedId = 0;
         this._topicChangedId = 0;
-        this._nicknameChangedId = 0;
+        this._membersChangedId = 0;
 
         this._room = room;
         this._revealer.reveal_child = room != null;
 
         this._updateTitlebar();
-        this._updateNick();
-        this._updateSensitivity();
-        this._updateCompletions();
 
         if (!this._room)
             return; // finished
@@ -308,44 +238,38 @@ const MainWindow = new Lang.Class({
                                Lang.bind(this, this._updateTitlebar));
         this._membersChangedId =
             this._room.connect('members-changed',
-                               Lang.bind(this, this._updateCompletions));
-        this._nicknameChangedId =
-            this._room.channel.connection.connect('notify::self-contact',
-                                                  Lang.bind(this,
-                                                            this._updateNick));
-
-        this._chatStack.set_visible_child_name(this._room.id);
+                               Lang.bind(this, this._updateUserListLabel));
     },
 
-    _setNick: function(nick) {
-        this._nickEntry.placeholder_text = nick;
+    _createWidget: function(app) {
+        let builder = new Gtk.Builder();
+        builder.add_from_resource('/org/gnome/polari/main-window.ui');
 
-        let account = this._room.channel.connection.get_account();
-        account.set_nickname_async(nick, Lang.bind(this,
-            function(a, res) {
-                try {
-                    a.set_nickname_finish(res);
-                } catch(e) {
-                    logError(e, "Failed to change nick");
+        this.window = builder.get_object('main_window');
+        this.window.application = app;
 
-                    this._updateNick();
-                    return;
-                }
+        let overlay = builder.get_object('overlay');
+        let sizeGroup = builder.get_object('bottom_size_group');
+        this._roomStack = new RoomStack.RoomStack(sizeGroup);
+        overlay.add(this._roomStack.widget);
 
-                // TpAccount:nickname is a local property which doesn't
-                // necessarily match the externally visible nick; telepathy
-                // doesn't consider failing to sync the two an error, so
-                // we give the server MAX_NICK_UPDATE_TIME seconds until
-                // we assume failure and revert back to the server nick
-                //
-                // (set_aliases() would do what we want, but it's not
-                // introspected)
-                Mainloop.timeout_add_seconds(MAX_NICK_UPDATE_TIME,
-                    Lang.bind(this, function() {
-                        this._updateNick();
-                        return false;
-                    }));
-            }));
+        this._titlebarRight = builder.get_object('titlebar_right');
+        this._titlebarLeft = builder.get_object('titlebar_left');
+
+        this._titleLabel = builder.get_object('title_label');
+        this._subtitleLabel = builder.get_object('subtitle_label');
+
+        this._selectionRevealer = builder.get_object('selection_toolbar_revealer');
+        this._joinMenuButton = builder.get_object('join_menu_button');
+        this._showUserListButton = builder.get_object('show_user_list_button');
+        this._revealer = builder.get_object('room_list_revealer');
+
+        let scroll = builder.get_object('room_list_scrollview');
+        this._roomList = new RoomList.RoomList();
+        scroll.add(this._roomList.widget);
+
+        this._userListPopover = new UserList.UserListPopover();
+        this._userListPopover.widget.relative_to = this._showUserListButton;
     },
 
     showJoinRoomDialog: function() {
@@ -368,15 +292,15 @@ const MainWindow = new Lang.Class({
             });
     },
 
-    _updateCompletions: function() {
-        let nicks = [];
+    _updateUserListLabel: function() {
+        let numMembers = 0;
 
         if (this._room &&
-            this._room.channel.has_interface(Tp.IFACE_CHANNEL_INTERFACE_GROUP)) {
-            let members = this._room.channel.group_dup_members_contacts();
-            nicks = members.map(function(member) { return member.alias; });
-        }
-        this._completion.setCompletions(nicks);
+            this._room.channel &&
+            this._room.channel.has_interface(Tp.IFACE_CHANNEL_INTERFACE_GROUP))
+            numMembers = this._room.channel.group_dup_members_contacts().length;
+
+        this._showUserListButton.label = '%d'.format(numMembers);
     },
 
     _updateTitlebar: function() {
@@ -398,20 +322,5 @@ const MainWindow = new Lang.Class({
         this._subtitleLabel.visible = subtitle.length > 0;
 
         this._titleLabel.label = this._room ? this._room.display_name : null;
-    },
-
-    _updateNick: function() {
-        let nick = this._room ? this._room.channel.connection.self_contact.alias
-                              : '';
-        this._nickEntry.placeholder_text = nick;
-    },
-
-    _updateSensitivity: function() {
-        this._inputArea.sensitive = this._room != null;
-
-        if (!this._inputArea.sensitive)
-            return;
-
-        this._entry.grab_focus();
     }
 });
