@@ -25,10 +25,15 @@ const RoomRow = new Lang.Class({
         this._selectionModeAction.connect('notify::state',
                           Lang.bind(this, this._onSelectionModeChanged));
 
-        room.channel.connect('message-received',
-                             Lang.bind(this, this._updatePending));
-        room.channel.connect('pending-message-removed',
-                             Lang.bind(this, this._updatePending));
+        room.connect('notify::channel', Lang.bind(this,
+            function() {
+                if (!room.channel)
+                    return;
+                room.channel.connect('message-received',
+                                     Lang.bind(this, this._updatePending));
+                room.channel.connect('pending-message-removed',
+                                     Lang.bind(this, this._updatePending));
+            }));
         room.bind_property('display-name', this._roomLabel, 'label',
                            GObject.BindingFlags.SYNC_CREATE);
 
@@ -47,14 +52,21 @@ const RoomRow = new Lang.Class({
     _updatePending: function() {
         let room = this.widget.room;
 
-        let pending = room.channel.dup_pending_messages();
+        let pending;
         let numPendingHighlights;
-        if (room.channel.has_interface(Tp.IFACE_CHANNEL_INTERFACE_GROUP))
-            numPendingHighlights = pending.filter(function(m) {
-                return room.should_highlight_message(m);
-            }).length;
-        else
-            numPendingHighlights = pending.length;
+
+        if (room.channel) {
+            pending = room.channel.dup_pending_messages();
+            if (room.channel.has_interface(Tp.IFACE_CHANNEL_INTERFACE_GROUP))
+                numPendingHighlights = pending.filter(function(m) {
+                    return room.should_highlight_message(m);
+                }).length;
+            else
+                numPendingHighlights = pending.length;
+        } else {
+            pending = [];
+            numPendingHighlights = 0;
+        }
 
         this._counter.label = numPendingHighlights.toString();
         this._counter.opacity = numPendingHighlights > 0 ? 1. : 0.;
@@ -184,6 +196,16 @@ const RoomList = new Lang.Class({
             function() {
                 this._moveSelection(Gtk.MovementStep.BUFFER_ENDS, 1);
             }));
+        action = app.lookup_action('nth-room');
+        action.connect('activate', Lang.bind(this,
+            function(action, param) {
+                let n = param.get_int32();
+                if (n > this._roomManager.roomCount)
+                    return;
+                this._moveSelection(Gtk.MovementStep.BUFFER_ENDS, -1);
+                if (n > 1)
+                    this._moveSelection(Gtk.MovementStep.DISPLAY_LINES, n - 1);
+            }));
     },
 
     _onSelectionModeChanged: function() {
@@ -211,27 +233,7 @@ const RoomList = new Lang.Class({
         let [id, ] = param.deep_unpack();
         let row = this._roomRows[id].widget;
 
-        let activeRoom = this._roomManager.getActiveRoom();
-        let current = this._roomRows[activeRoom.id].widget;
-
-        if (current == row) {
-            let selected = this.widget.get_selected_row();
-            let newActive = null;
-            let visibleChildren = this.widget.get_children().filter(function(c) {
-                return c.visible;
-            });
-            if (visibleChildren.length > 1) {
-                row.can_focus = false;
-                this.widget.select_row(row);
-                row.can_focus = true;
-                let count = row.get_index() == 0 ? 1 : -1;
-                this._moveSelection(Gtk.MovementStep.DISPLAY_LINES, count);
-                newActive = this.widget.get_selected_row().room;
-            }
-            this._roomManager.setActiveRoom(newActive);
-            if (selected != row)
-                this.widget.select_row(selected);
-        }
+        this._moveSelectionFromRow(row);
         row.hide();
     },
 
@@ -249,6 +251,31 @@ const RoomList = new Lang.Class({
             focus.emit('grab-focus');
     },
 
+    _moveSelectionFromRow: function(row) {
+        let activeRoom = this._roomManager.getActiveRoom();
+        let current = this._roomRows[activeRoom.id].widget;
+
+        if (current != row)
+            return;
+
+        let selected = this.widget.get_selected_row();
+        let newActive = null;
+        let visibleChildren = this.widget.get_children().filter(function(c) {
+            return c.visible;
+        });
+        if (visibleChildren.length > 1) {
+            row.can_focus = false;
+            this.widget.select_row(row);
+            row.can_focus = true;
+            let count = row.get_index() == 0 ? 1 : -1;
+            this._moveSelection(Gtk.MovementStep.DISPLAY_LINES, count);
+            newActive = this.widget.get_selected_row().room;
+        }
+        this._roomManager.setActiveRoom(newActive);
+        if (selected != row)
+            this.widget.select_row(selected);
+    },
+
     _roomAdded: function(roomManager, room) {
         let roomRow = new RoomRow(room);
         this.widget.add(roomRow.widget);
@@ -276,6 +303,7 @@ const RoomList = new Lang.Class({
         if (!roomRow)
             return;
 
+        this._moveSelectionFromRow(roomRow.widget);
         this.widget.remove(roomRow.widget);
         delete this._roomRows[room.id];
     },
@@ -308,7 +336,7 @@ const RoomList = new Lang.Class({
 
     _updateHeader: function(row, before) {
         let getAccount = function(row) {
-            return row ? row.room.channel.connection.get_account() : null;
+            return row ? row.room.account : null;
         };
         let beforeAccount = getAccount(before);
         let account = getAccount(row);
