@@ -8,12 +8,19 @@ const Tp = imports.gi.TelepathyGLib;
 const ChatroomManager = imports.chatroomManager;
 const Lang = imports.lang;
 
-const UserListSidebar = new Lang.Class({
-    Name: 'UserListSidebar',
+const UserListPopover = new Lang.Class({
+    Name: 'UserListPopover',
 
     _init: function() {
         this._createWidget();
 
+        this.widget.connect('map', Lang.bind(this, function() {
+            this._revealer.transition_duration = 0;
+        }));
+        this._revealer.connect('notify::child-revealed', Lang.bind(this, function() {
+            this._revealer.transition_duration = 250;
+        }));
+
         this._rooms = {};
         this._room = null;
 
@@ -26,24 +33,29 @@ const UserListSidebar = new Lang.Class({
                                   Lang.bind(this, this._activeRoomChanged));
     },
 
-    set animateEntry(animate) {
-        this._revealer.transition_duration = animate ? 250 : 0;
-    },
-
     _createWidget: function() {
-        this.widget = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL });
+        this.widget = new Gtk.Popover({ modal: true,
+                                        position: Gtk.PositionType.TOP,
+                                        vexpand: true,
+                                        margin_left: 12,
+                                        margin_right: 12,
+                                        margin_bottom: 12 });
+        this.widget.set_border_width(6);
+        this.widget.set_size_request(250, -1);
+
+        this.widget.get_style_context().add_class('polari-user-list');
+
+        this._box = new Gtk.Box({ orientation: Gtk.Orientation.VERTICAL,
+                                  spacing: 6 });
+        this.widget.add(this._box);
 
         this._revealer = new Gtk.Revealer();
-        this.widget.add(this._revealer);
+        this._box.add(this._revealer);
 
-        let frame = new Gtk.Frame();
-        frame.get_style_context().add_class('polari-user-list-search-area');
-        this._revealer.add(frame);
-
-        this._entry = new Gtk.SearchEntry({ margin: 4 });
+        this._entry = new Gtk.SearchEntry();
         this._entry.connect('search-changed',
                             Lang.bind(this, this._updateFilter));
-        this._entry.connect_after('key-press-event', Lang.bind(this,
+        this._entry.connect('key-press-event', Lang.bind(this,
             function(w, event) {
                 let [, keyval] = event.get_keyval();
                 if (keyval == Gdk.KEY_Escape) {
@@ -52,24 +64,18 @@ const UserListSidebar = new Lang.Class({
                 }
                 return false;
             }));
-        frame.add(this._entry);
+        this._revealer.add(this._entry);
 
-        this._stack = new Gtk.Stack({ hexpand: true, vexpand: true });
-        this._stack.transition_type = Gtk.StackTransitionType.CROSSFADE;
-        this.widget.add(this._stack);
-
-        this.widget.show_all();
+        this._box.show_all();
     },
 
     _roomAdded: function(roomManager, room) {
-        if (room.channel.handle_type != Tp.HandleType.ROOM)
+        if (room.type != Tp.HandleType.ROOM)
             return;
 
         let userList = new UserList(room);
         this._rooms[room.id] = userList;
 
-        this._stack.add_named(userList.widget, room.id);
-
         userList.widget.vadjustment.connect('changed',
                                             Lang.bind(this, this._updateEntryVisibility));
     },
@@ -85,12 +91,16 @@ const UserListSidebar = new Lang.Class({
         this._entry.text = '';
         this._updateFilter();
 
+        let currentList = this._room ? this._rooms[this._room.id] : null;
+        if (currentList)
+            this._box.remove(currentList.widget);
+
         this._room = room;
 
         if (!room || !this._rooms[room.id])
             return;
 
-        this._stack.set_visible_child_name(room.id);
+        this._box.add(this._rooms[room.id].widget);
         this._updateEntryVisibility();
     },
 
@@ -99,7 +109,7 @@ const UserListSidebar = new Lang.Class({
             return;
         let userList = this._rooms[this._room.id];
         let [, natHeight] = userList.widget.get_child().get_preferred_height();
-        let height = this.widget.get_allocated_height();
+        let height = userList.widget.get_allocated_height();
         this._revealer.reveal_child = this._entry.text != '' ||
                                       natHeight > height;
     },
@@ -310,14 +320,16 @@ const UserList = new Lang.Class({
     Name: 'UserList',
 
     _init: function(room) {
-        this.widget = new Gtk.ScrolledWindow();
+        this.widget = new Gtk.ScrolledWindow({ hexpand: true, vexpand: true,
+                                               shadow_type: Gtk.ShadowType.ETCHED_IN });
         this.widget.hscrollbar_policy = Gtk.PolicyType.NEVER;
 
         this._list = new Gtk.ListBox();
         this.widget.add(this._list);
 
         this._list.set_selection_mode(Gtk.SelectionMode.NONE);
-        this._list.set_header_func(Lang.bind(this, this._updateHeader));
+        /* see https://bugzilla.gnome.org/show_bug.cgi?id=725403 */
+        //this._list.set_header_func(Lang.bind(this, this._updateHeader));
         this._list.set_filter_func(Lang.bind(this, this._filterRows));
         this._list.set_sort_func(Lang.bind(this, this._sort));
 
@@ -340,12 +352,13 @@ const UserList = new Lang.Class({
                      Lang.bind(this, this._onMemberRemoved));
         room.connect('member-joined',
                      Lang.bind(this, this._onMemberJoined));
+        /*
+        // see https://bugzilla.gnome.org/show_bug.cgi?id=725403
         room.connect('members-changed',
                      Lang.bind(this, this._onMembersChanged));
-
-        let members = room.channel.group_dup_members_contacts();
-        for (let i = 0; i < members.length; i++)
-            this._addMember(members[i]);
+        */
+        room.connect('notify::channel',
+                     Lang.bind(this, this._onChannelChanged));
 
         this.widget.show_all();
     },
@@ -373,6 +386,18 @@ const UserList = new Lang.Class({
         this._counterLabel.label = numMembers.toString();
     },
 
+    _onChannelChanged: function(room) {
+        this._list.foreach(function(w) { w.destroy(); });
+        this._rows = {};
+
+        if (!room.channel)
+            return;
+
+        let members = room.channel.group_dup_members_contacts();
+        for (let i = 0; i < members.length; i++)
+            this._addMember(members[i]);
+    },
+
     _addMember: function(member) {
         let row = new UserListRow(member);
         this._rows[member] = row;
@@ -416,6 +441,9 @@ const UserList = new Lang.Class({
         if (row.get_header())
             return;
 
+        if (!this._room.channel)
+            return;
+
         let members = this._room.channel.group_dup_members_contacts();
         let numMembers = members.length;
 
